@@ -260,10 +260,6 @@ void DestroyPicBuff (PWelsDecoderContext pCtx, PPicBuff* ppPicBuf, CMemoryAlign*
 
   ResetReorderingPictureBuffers (pCtx->pPictReoderingStatus, pCtx->pPictInfoList, false);
 
-  // Valgrind on Linux says this an invalid write, and we have spurious
-  // crashes on MacOS. Disabling this line resolves both.
-  // if (pCtx->pDstInfo) pCtx->pDstInfo->iBufferStatus = 0;
-
   if (NULL == ppPicBuf || NULL == *ppPicBuf)
     return;
 
@@ -308,6 +304,7 @@ void ResetReorderingPictureBuffers (PPictReoderingStatus pPictReoderingStatus, P
       pPictInfo[i].iPOC = IMinInt32;
     }
     pPictInfo->sBufferInfo.iBufferStatus = 0;
+		pPictReoderingStatus->bHasBSlice = false;
   }
 }
 
@@ -445,7 +442,8 @@ static inline int32_t GetTargetRefListSize (PWelsDecoderContext pCtx) {
     iNumRefFrames = pCtx->pSps->iNumRefFrames + 2;
     int32_t  iThreadCount = GetThreadCount (pCtx);
     if (iThreadCount > 1) {
-      iNumRefFrames = MAX_REF_PIC_COUNT;
+      //due to thread and reordering buffering, it needs more dpb space
+      iNumRefFrames = MAX_DPB_COUNT + iThreadCount;
     }
   }
 
